@@ -6,7 +6,7 @@
 #include "pdf_fncs.h"
 #include "cdf_fncs.h"
 #include "fncs_seven.h"
-#include "cubature.h"
+#include "gauss.h"
 
 
 
@@ -32,18 +32,21 @@ int int_ddiff(unsigned dim, const double *x, void *p, unsigned fdim, double *ret
 	// double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	//double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+	//double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	// usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t - tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t - tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = -0.5 * pow(y, 2) - M_LN_SQRT_PI - 0.5 * M_LN2 + log1p(temp) - 2 * log1p(-temp);
+		//if (sv) temp2 = -0.5 * pow(y, 2) - M_LN_SQRT_PI - 0.5 * M_LN2 + log1p(temp) - 2 * log1p(-temp);
 
 		double integrand = exp(ldW + temp2);
 
@@ -70,20 +73,23 @@ int int_daddiff(unsigned dim, const double* x, void* p, unsigned fdim, double* r
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	//double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+	//double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	// usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
-		dadwiener(low_or_up * (t-tau), a, nu, omega, ldW, val_ptr, errorW, K, epsFLAG);
+		dadwiener(low_or_up * (t-tau), a, v, omega, sv, ldW, val_ptr, errorW, K, epsFLAG);
 		double dda = val_ptr[0];
 
 		double integrand = dda * exp(temp2);
@@ -111,20 +117,23 @@ int int_dvddiff(unsigned dim, const double* x, void* p, unsigned fdim, double* r
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	//double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+	//double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	// usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
-		dvdwiener(low_or_up * (t-tau), a, nu, omega, ldW, val_ptr);
+		dvdwiener(low_or_up * (t-tau), a, v, omega, sv, ldW, val_ptr);
 
 		double integrand = val_ptr[0] * exp(temp2);
 
@@ -151,23 +160,26 @@ int int_dt0ddiff(unsigned dim, const double* x, void* p, unsigned fdim, double*
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	//double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+	//double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	// usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
 		double wn = omega;
 		if (low_or_up==1) wn = 1-omega;
 
-		dtdwiener(t-tau, a, -low_or_up*nu, wn, ldW, val_ptr, errorW, K, epsFLAG);
+		dtdwiener(t-tau, a, -low_or_up*v, wn, sv, ldW, val_ptr, errorW, K, epsFLAG);
 
 		double integrand = -val_ptr[0] * exp(temp2);
 
@@ -194,20 +206,23 @@ int int_dwddiff(unsigned dim, const double* x, void* p, unsigned fdim, double* r
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	//double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+	//double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	// usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
-		dwdwiener(low_or_up * (t-tau), a, nu, omega, ldW, val_ptr, errorW, K, epsFLAG);
+		dwdwiener(low_or_up * (t-tau), a, v, omega, sv, ldW, val_ptr, errorW, K, epsFLAG);
 
 		double integrand = val_ptr[0] * exp(temp2);
 
@@ -234,21 +249,21 @@ int int_dswddiff(unsigned dim, const double* x, void* p, unsigned fdim, double*
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? w + sw * (x[1] - 0.5) : w + sw * (x[0] - 0.5);
-	double tau = sv ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0);
-	double temp_sw = sv ? (x[1]-0.5) : (x[0]-0.5);
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	double omega = w + sw * (x[0] - 0.5);
+	double tau = st ? t0 + st * x[1] : t0;
+	double temp_sw = x[0]-0.5;
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
-		dwdwiener(low_or_up * (t-tau), a, nu, omega, ldW, val_ptr, errorW, K, epsFLAG);
+		dwdwiener(low_or_up * (t-tau), a, v, omega, sv, ldW, val_ptr, errorW, K, epsFLAG);
 
 		double integrand = temp_sw * val_ptr[0] * exp(temp2);
 
@@ -275,19 +290,22 @@ int int_dsvddiff(unsigned dim, const double* x, void* p, unsigned fdim, double*
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = pow(x[0], 2);
-	double y = x[0] / (1 - temp);
-	double nu = v + sv * y;
-	double omega = sw ? w + sw * (x[1] - 0.5) : w;
-	double tau = sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0);
+	//double temp = pow(x[0], 2);
+	//double y = x[0] / (1 - temp);
+	//double nu = v + sv * y;
+	double y = 1;
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
-		double temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+	  double temp2 = 0;
+		//temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
-		dvdwiener(low_or_up * (t-tau), a, nu, omega, ldW, val_ptr);
+		//dvdwiener(low_or_up * (t-tau), a, v, omega, sv, ldW, val_ptr);
+		dsvdwiener(low_or_up * (t-tau), a, v, omega, sv, ldW, val_ptr, errorW, K, epsFLAG);
 
 		double integrand = y * val_ptr[0] * exp(temp2);
 
@@ -314,24 +332,24 @@ int int_dst0ddiff(unsigned dim, const double* x, void* p, unsigned fdim, double*
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
-	double temp_st0 = sw ? (sv ? -x[2] : -x[1]) : (sv ? -x[1] : -x[0]);
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
+	double temp_st0 = sw ? -x[1] : -x[0];
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
 		double wn = omega;
 		if (low_or_up==1) wn = 1-omega;
 
-		dtdwiener(t-tau, a, -low_or_up*nu, wn, ldW, val_ptr, errorW, K, epsFLAG);
+		dtdwiener(t-tau, a, -low_or_up*v, wn, sv, ldW, val_ptr, errorW, K, epsFLAG);
 
 		double integrand = temp_st0 * val_ptr[0] * exp(temp2);
 
@@ -358,23 +376,26 @@ int int_dtddiff(unsigned dim, const double* x, void* p, unsigned fdim, double* r
 	double *val_ptr = (params->val_ptr);
 
 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	//double temp = sv ? pow(x[0], 2) : 0;
+	//double y = sv ? x[0] / (1 - temp) : 0;
+	//double nu = sv ? v + sv * y : v;
+	//double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+	//double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+	// usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
 	if (t - tau <= 0) retval[0] = 0.0;
 	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
 
 		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
 
 		double wn = omega;
 		if (low_or_up==1) wn = 1-omega;
 
-		dtdwiener(t-tau, a, -low_or_up*nu, wn, ldW, val_ptr, errorW, K, epsFLAG);
+		dtdwiener(t-tau, a, -low_or_up*v, wn, sv, ldW, val_ptr, errorW, K, epsFLAG);
 
 		double integrand = val_ptr[0] * exp(temp2);
 
@@ -383,6 +404,47 @@ int int_dtddiff(unsigned dim, const double* x, void* p, unsigned fdim, double* r
 	return 0;
 }
 
+
+/* integrand d/dst0 ___CDF___ */
+int int_dst0pdiff(unsigned dim, const double* x, void* p, unsigned fdim, double* retval) {
+  my_params *params = static_cast<my_params*>(p);
+  double t = (params->t);
+  int low_or_up = (params->low_or_up);
+  double a = (params->a);
+  double v = (params->v);
+  double t0 = (params->t0);
+  double w = (params->w);
+  double sw = (params->sw);
+  double sv = (params->sv);
+  double st = (params->st);
+  double errorW = (params->errorW);
+  int K = (params->K);
+  int epsFLAG = (params->epsFLAG);
+  // double *val_ptr = (params->val_ptr);
+  
+  // usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
+  //double temp = sv ? pow(x[0], 2) : 0;
+  //double y = sv ? x[0] / (1 - temp) : 0;
+  //double nu = sv ? v + sv * y : v;
+  double omega = sw ? w + sw * (x[0] - 0.5) : w;
+  double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
+  double temp_st0 = sw ? -x[1] : -x[0];
+  
+  if (t - tau <= 0) retval[0] = 0.0;
+  else {
+    double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
+    
+    double temp2 = 0;
+    //if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+    
+    double integrand = temp_st0 * exp(ldW + temp2);
+    
+    retval[0] = integrand;
+  }
+  return 0;
+}
+
+
 /* calculate density for 7-param diffusion */
 void ddiff(int choice, double t, int low_or_up, double a, double v, double t0, double w, double sw, double sv, double st, double myerr, int K, int epsFLAG, int Neval, double *derivF, double *Rerr) {
 
@@ -397,24 +459,24 @@ void ddiff(int choice, double t, int low_or_up, double a, double v, double t0, d
 
 	my_params params = {t, low_or_up, a, v, t0, w, sw, sv, st, errorW, K, epsFLAG, val_ptr};
 
-	int dim = (sw!=0)+(sv!=0)+(st!=0);
+	int dim = (sw!=0)+(st!=0);
 
-	double *xmin = (double*)malloc(dim * sizeof(double));
-	double *xmax = (double*)malloc(dim * sizeof(double));
+	double *xmin = (double*)R_Calloc(dim, double);
+	double *xmax = (double*)R_Calloc(dim, double);
 
 	// 0  = s (v); 1 = u (w), 2 = v (w)
-	if(sv) {
-		xmin[0] = -1; xmax[0] = 1;
-		for (int i = 1; i < dim; i++) {
-			xmin[i] = 0;
-			xmax[i] = 1;
-		}
-	} else {
-		for (int i = 0; i < dim; i++) {
-			xmin[i] = 0;
-			xmax[i] = 1;
-		}
+//	if(sv) {
+//		xmin[0] = -1; xmax[0] = 1;
+//		for (int i = 1; i < dim; i++) {
+//			xmin[i] = 0;
+//			xmax[i] = 1;
+//		}
+//	} else {
+	for (int i = 0; i < dim; i++) {
+		xmin[i] = 0;
+		xmax[i] = 1;
 	}
+//	}
 	if (st) xmax[dim-1] = fmin(1.0, (t-t0)/st);
 
 	double reltol = 0.0;
@@ -426,19 +488,20 @@ void ddiff(int choice, double t, int low_or_up, double a, double v, double t0, d
 
 	//	printf("%u-dim integral, tolerance = %g\n", dim, tol);
 	switch (choice) {
-		case 0: hcubature(1, int_ddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-	  case 1: hcubature(1, int_daddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 2: hcubature(1, int_dvddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 3: hcubature(1, int_dt0ddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 4: hcubature(1, int_dwddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 5: hcubature(1, int_dswddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 6: hcubature(1, int_dsvddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 7: hcubature(1, int_dst0ddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 8: hcubature(1, int_dtddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
+		case 0: hcubature(int_ddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+	  case 1: hcubature(int_daddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 2: hcubature(int_dvddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 3: hcubature(int_dt0ddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 4: hcubature(int_dwddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 5: hcubature(int_dswddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 6: hcubature(int_dsvddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 7: hcubature(int_dst0ddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 8: hcubature(int_dtddiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+	  case 9: hcubature(int_dst0pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
 	}
 	//if(err > abstol) Rprintf("absolute error not achieved: %g < %g\n", abstol, err);
 
-	free(xmin); free(xmax);
+	R_Free(xmin); R_Free(xmax);
 	*derivF = val;
 	if (*Rerr < err+errorW) *Rerr = err+errorW;
 
@@ -577,42 +640,45 @@ int int_dvpdiff(unsigned dim, const double* x, void* p, unsigned fdim, double* r
 	return 0;
 }
 
-/* integrand d/dt0 */
+/* integrand d/dt0 */ // not needed anymore. Can be done by just taking -PDF
 int int_dt0pdiff(unsigned dim, const double* x, void* p, unsigned fdim, double* retval) {
-	my_params *params = static_cast<my_params*>(p);
-	double t = (params->t);
-	int low_or_up = (params->low_or_up);
-	double a = (params->a);
-	double v = (params->v);
-	double t0 = (params->t0);
-	double w = (params->w);
-	double sw = (params->sw);
-	double sv = (params->sv);
-	double st = (params->st);
-	double errorW = (params->errorW);
-	int K = (params->K);
-	int epsFLAG = (params->epsFLAG);
-	// double *val_ptr = (params->val_ptr);
+  my_params *params = static_cast<my_params*>(p);
+  double t = (params->t);
+  int low_or_up = (params->low_or_up);
+  double a = (params->a);
+  double v = (params->v);
+  double t0 = (params->t0);
+  double w = (params->w);
+  double sw = (params->sw);
+  double sv = (params->sv);
+  double st = (params->st);
+  double errorW = (params->errorW);
+  int K = (params->K);
+  int epsFLAG = (params->epsFLAG);
+  // double *val_ptr = (params->val_ptr);
 
-	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+  // usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
+  //double temp = sv ? pow(x[0], 2) : 0;
+  //double y = sv ? x[0] / (1 - temp) : 0;
+  //double nu = sv ? v + sv * y : v;
+  //double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
+  //double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
+  // usually: 0  = omega (w), 1 = tau (t0), depending on whether sv, sw, or st = 0
+  double omega = sw ? w + sw * (x[0] - 0.5) : w;
+  double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
 
-	if (t - tau <= 0) retval[0] = 0.0;
-	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
+  if (t - tau <= 0) retval[0] = 0.0;
+  else {
+    double ldW = dwiener(low_or_up * (t - tau), a, v, omega, sv, errorW, K, epsFLAG);
 
-		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+    double temp2 = 0;
+    //if (sv) temp2 = -0.5 * pow(y, 2) - M_LN_SQRT_PI - 0.5 * M_LN2 + log1p(temp) - 2 * log1p(-temp);
 
-		double integrand = -exp(ldW + temp2);
+    double integrand = -exp(ldW + temp2);
 
-		retval[0] = integrand;
-	}
-	return 0;
+    retval[0] = integrand;
+  }
+  return 0;
 }
 
 /* integrand d/dz */
@@ -746,44 +812,44 @@ int int_dsvpdiff(unsigned dim, const double* x, void* p, unsigned fdim, double*
 	return 0;
 }
 
-/* integrand d/dst0 */
-int int_dst0pdiff(unsigned dim, const double* x, void* p, unsigned fdim, double* retval) {
-	my_params *params = static_cast<my_params*>(p);
-	double t = (params->t);
-	int low_or_up = (params->low_or_up);
-	double a = (params->a);
-	double v = (params->v);
-	double t0 = (params->t0);
-	double w = (params->w);
-	double sw = (params->sw);
-	double sv = (params->sv);
-	double st = (params->st);
-	double errorW = (params->errorW);
-	int K = (params->K);
-	int epsFLAG = (params->epsFLAG);
-	// double *val_ptr = (params->val_ptr);
-
-	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
-	double temp = sv ? pow(x[0], 2) : 0;
-	double y = sv ? x[0] / (1 - temp) : 0;
-	double nu = sv ? v + sv * y : v;
-	double omega = sv ? (sw ? w + sw * (x[1] - 0.5) : w) : (sw ? w + sw * (x[0] - 0.5) : w);
-	double tau = sv ? ( sw ? (st ? t0 + st * x[2] : t0) : (st ? t0 + st * x[1] : t0) ) : ( sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0) );
-	double temp_st0 = sw ? (sv ? -x[2] : -x[1]) : (sv ? -x[1] : -x[0]);
-
-	if (t - tau <= 0) retval[0] = 0.0;
-	else {
-		double ldW = dwiener(low_or_up * (t-tau), a, nu, omega, errorW, K, epsFLAG);
-
-		double temp2 = 0;
-		if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
-
-		double integrand = temp_st0 * exp(ldW + temp2);
-
-		retval[0] = integrand;
-	}
-	return 0;
-}
+// /* integrand d/dst0 (see above) */
+// int int_dst0pdiff(unsigned dim, const double* x, void* p, unsigned fdim, double* retval) {
+// 	my_params *params = static_cast<my_params*>(p);
+// 	double t = (params->t);
+// 	int low_or_up = (params->low_or_up);
+// 	double a = (params->a);
+// 	double v = (params->v);
+// 	double t0 = (params->t0);
+// 	double w = (params->w);
+// 	double sw = (params->sw);
+// 	double sv = (params->sv);
+// 	double st = (params->st);
+// 	double errorW = (params->errorW);
+// 	int K = (params->K);
+// 	int epsFLAG = (params->epsFLAG);
+// 	// double *val_ptr = (params->val_ptr);
+// 
+// 	// usually: 0  = s (v); 1 = u (w), 2 = v (t), depending on whether sv, sw, or st = 0
+// 	//double temp = sv ? pow(x[0], 2) : 0;
+// 	//double y = sv ? x[0] / (1 - temp) : 0;
+// 	//double nu = sv ? v + sv * y : v;
+// 	double omega = sw ? w + sw * (x[0] - 0.5) : w;
+// 	double tau = sw ? (st ? t0 + st * x[1] : t0) : (st ? t0 + st * x[0] : t0);
+// 	double temp_st0 = sw ? -x[1] : -x[0];
+// 
+// 	if (t - tau <= 0) retval[0] = 0.0;
+// 	else {
+// 		double ldW = dwiener(low_or_up * (t-tau), a, v, omega, sv, errorW, K, epsFLAG);
+// 
+// 		double temp2 = 0;
+// 		//if (sv) temp2 = - 0.5*pow(y, 2) - M_LN_SQRT_PI - 0.5*M_LN2 + log1p(temp) - 2*log1p(-temp);
+// 
+// 		double integrand = temp_st0 * exp(ldW + temp2);
+// 
+// 		retval[0] = integrand;
+// 	}
+// 	return 0;
+// }
 
 /* calculate distribution for 7-param diffusion */
 void pdiff(int choice, double t, int low_or_up, double a, double v, double t0, double w, double sw, double sv, double st, double myerr, int K, int epsFLAG, int Neval, double *derivF, double *Rerr) {
@@ -801,8 +867,8 @@ void pdiff(int choice, double t, int low_or_up, double a, double v, double t0, d
 
 	int dim = (sw!=0)+(sv!=0)+(st!=0);
 
-	double *xmin = (double*)malloc(dim * sizeof(double));
-	double *xmax = (double*)malloc(dim * sizeof(double));
+	double *xmin = (double*)R_Calloc(dim, double);
+	double *xmax = (double*)R_Calloc(dim, double);
 
 	// 0  = s (v); 1 = u (w), 2 = v (w)
 	if(sv) {
@@ -828,18 +894,18 @@ void pdiff(int choice, double t, int low_or_up, double a, double v, double t0, d
 
 	//	printf("%u-dim integral, tolerance = %g\n", dim, tol);
 	switch (choice) {
-		case 0: hcubature(1, int_pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-	  case 1: hcubature(1, int_dapdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 2: hcubature(1, int_dvpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 3: hcubature(1, int_dt0pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 4: hcubature(1, int_dwpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 5: hcubature(1, int_dswpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 6: hcubature(1, int_dsvpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
-		case 7: hcubature(1, int_dst0pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
+		case 0: hcubature(int_pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+	  case 1: hcubature(int_dapdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 2: hcubature(int_dvpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 3: hcubature(int_dt0pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 4: hcubature(int_dwpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 5: hcubature(int_dswpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		case 6: hcubature(int_dsvpdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, &val, &err); break;
+		//case 7: hcubature(1, int_dst0pdiff, &params, dim, xmin, xmax, Meval, abstol, reltol, ERROR_INDIVIDUAL, &val, &err); break;
 	}
 	//if(err > abstol) Rprintf("absolute error not achieved: %g < %g\n", abstol, err);
 
-	free(xmin); free(xmax);
+	R_Free(xmin); R_Free(xmax);
 	*derivF = val;
 	if (*Rerr < err+errorW) *Rerr = err+errorW;
 
